@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2008 Massachusetts Institute of Technology.
+/* Copyright (C) 2005-2014 Massachusetts Institute of Technology.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,11 +23,11 @@
 namespace meep {
 
 struct integrate_data {
-  int num_fields;
+  int num_fvals;
   const component *components;
   component *cS;
   complex<double> *ph;
-  complex<double> *fields;
+  complex<double> *fvals;
   int *offsets;
   int ninveps;
   component inveps_cs[3];
@@ -53,7 +53,7 @@ static void integrate_chunkloop(fields_chunk *fc, int ichunk, component cgrid,
   integrate_data *data = (integrate_data *) data_;
   int *off = data->offsets;
   component *cS = data->cS;
-  complex<double> *fields = data->fields, *ph = data->ph;
+  complex<double> *fvals = data->fvals, *ph = data->ph;
   complex<long double> sum = 0.0;
   double maxabs = 0;
   const component *iecs = data->inveps_cs;
@@ -63,45 +63,46 @@ static void integrate_chunkloop(fields_chunk *fc, int ichunk, component cgrid,
   const direction *imds = data->invmu_ds;
   int imos[6];
 
-  for (int i = 0; i < data->num_fields; ++i) {
+  for (int i = 0; i < data->num_fvals; ++i) {
     cS[i] = S.transform(data->components[i], -sn);
     if (cS[i] == Dielectric || cS[i] == Permeability)
       ph[i] = 1.0;
     else {
-      if (cgrid == Dielectric)
-	fc->v.yee2diel_offsets(cS[i], off[2*i], off[2*i+1]);
+      if (cgrid == Centered)
+	fc->gv.yee2cent_offsets(cS[i], off[2*i], off[2*i+1]);
       ph[i] = shift_phase * S.phase_shift(cS[i], sn);
     }
   }
   for (int k = 0; k < data->ninveps; ++k)
-    fc->v.yee2diel_offsets(iecs[k], ieos[2*k], ieos[2*k+1]);
+    fc->gv.yee2cent_offsets(iecs[k], ieos[2*k], ieos[2*k+1]);
   for (int k = 0; k < data->ninvmu; ++k)
-    fc->v.yee2diel_offsets(imcs[k], imos[2*k], imos[2*k+1]);
+    fc->gv.yee2cent_offsets(imcs[k], imos[2*k], imos[2*k+1]);
 
-  vec rshift(shift * (0.5*fc->v.inva));
-  LOOP_OVER_IVECS(fc->v, is, ie, idx) {
-    IVEC_LOOP_LOC(fc->v, loc);
+  vec rshift(shift * (0.5*fc->gv.inva));
+  LOOP_OVER_IVECS(fc->gv, is, ie, idx) {
+    IVEC_LOOP_LOC(fc->gv, loc);
     loc = S.transform(loc, sn) + rshift;
 
-    for (int i = 0; i < data->num_fields; ++i) {
+    for (int i = 0; i < data->num_fvals; ++i) {
       if (cS[i] == Dielectric) {
 	double tr = 0.0;
-	for (int k = 0; k < data->ninveps; ++k)
-	  tr += (fc->s->inveps[iecs[k]][ieds[k]][idx]
-		 + fc->s->inveps[iecs[k]][ieds[k]][idx+ieos[2*k]]
-		 + fc->s->inveps[iecs[k]][ieds[k]][idx+ieos[1+2*k]]
-		 + fc->s->inveps[iecs[k]][ieds[k]][idx+ieos[2*k]+ieos[1+2*k]]);
-	fields[i] = (4 * data->ninveps) / tr;
+	for (int k = 0; k < data->ninveps; ++k) {
+	  const realnum *ie = fc->s->chi1inv[iecs[k]][ieds[k]];
+	  if (ie) tr += (ie[idx] + ie[idx+ieos[2*k]] + ie[idx+ieos[1+2*k]]
+			 + ie[idx+ieos[2*k]+ieos[1+2*k]]);
+	  else tr += 4; // default inveps == 1
+	}
+	fvals[i] = (4 * data->ninveps) / tr;
       }
       else if (cS[i] == Permeability) {
 	double tr = 0.0;
 	for (int k = 0; k < data->ninvmu; ++k) {
-	  const double *im = fc->s->invmu[imcs[k]][imds[k]];
+	  const realnum *im = fc->s->chi1inv[imcs[k]][imds[k]];
 	  if (im) tr += (im[idx] + im[idx+imos[2*k]] + im[idx+imos[1+2*k]]
 			 + im[idx+imos[2*k]+imos[1+2*k]]);
 	  else tr += 4; // default invmu == 1
 	}
-	fields[i] = (4 * data->ninvmu) / tr;
+	fvals[i] = (4 * data->ninvmu) / tr;
       }
       else {
 	double f[2];
@@ -113,12 +114,12 @@ static void integrate_chunkloop(fields_chunk *fc, int ichunk, component cgrid,
 			   + fc->f[cS[i]][k][idx+off[2*i]+off[2*i+1]]);
 	  else
 	    f[k] = 0;
-	fields[i] = complex<double>(f[0], f[1]) * ph[i];
+	fvals[i] = complex<double>(f[0], f[1]) * ph[i];
       }
     }
 
     complex<double> integrand = 
-      data->integrand(fields, loc, data->integrand_data_);
+      data->integrand(fvals, loc, data->integrand_data_);
     maxabs = max(maxabs, abs(integrand));
     sum += integrand * IVEC_LOOP_WEIGHT(s0, s1, e0, e1, dV0 + dV1 * loop_i2);
   }
@@ -127,30 +128,30 @@ static void integrate_chunkloop(fields_chunk *fc, int ichunk, component cgrid,
   data->sum += sum;
 }
 
-complex<double> fields::integrate(int num_fields, const component *components,
+complex<double> fields::integrate(int num_fvals, const component *components,
 				  field_function integrand,
 				  void *integrand_data_,
-				  const geometric_volume &where,
+				  const volume &where,
 				  double *maxabs)
 {
   // check if components are all on the same grid:
   bool same_grid = true;
-  for (int i = 1; i < num_fields; ++i)
-    if (v.iyee_shift(components[i]) != v.iyee_shift(components[0])) {
+  for (int i = 1; i < num_fvals; ++i)
+    if (gv.iyee_shift(components[i]) != gv.iyee_shift(components[0])) {
       same_grid = false;
       break;
     }
 
-  component cgrid = Dielectric;
-  if (same_grid && num_fields > 0)
+  component cgrid = Centered;
+  if (same_grid && num_fvals > 0)
     cgrid = components[0];
 
   integrate_data data;
-  data.num_fields = num_fields;
+  data.num_fvals = num_fvals;
   data.components = components;
-  data.cS = new component[num_fields];
-  data.ph = new complex<double>[num_fields];
-  data.fields = new complex<double>[num_fields];
+  data.cS = new component[num_fvals];
+  data.ph = new complex<double>[num_fvals];
+  data.fvals = new complex<double>[num_fvals];
   data.sum = 0;
   data.maxabs = 0;
   data.integrand = integrand;
@@ -159,37 +160,37 @@ complex<double> fields::integrate(int num_fields, const component *components,
   /* compute inverse-epsilon directions for computing Dielectric fields */
   data.ninveps = 0;
   bool needs_dielectric = false;
-  for (int i = 0; i < num_fields; ++i)
+  for (int i = 0; i < num_fvals; ++i)
     if (components[i] == Dielectric) { needs_dielectric = true; break; }
   if (needs_dielectric) 
-    FOR_ELECTRIC_COMPONENTS(c) if (v.has_field(c)) {
-      if (data.ninvmu == 3) abort("more than 3 field components??");
-      data.invmu_cs[data.ninvmu] = c;
-      data.invmu_ds[data.ninvmu] = component_direction(c);
-      ++data.ninvmu;
+    FOR_ELECTRIC_COMPONENTS(c) if (gv.has_field(c)) {
+      if (data.ninveps == 3) abort("more than 3 field components??");
+      data.inveps_cs[data.ninveps] = c;
+      data.inveps_ds[data.ninveps] = component_direction(c);
+      ++data.ninveps;
     }
   
   /* compute inverse-mu directions for computing Permeability fields */
   data.ninvmu = 0;
   bool needs_permeability = false;
-  for (int i = 0; i < num_fields; ++i)
+  for (int i = 0; i < num_fvals; ++i)
     if (components[i] == Permeability) { needs_permeability = true; break; }
   if (needs_permeability) 
-    FOR_MAGNETIC_COMPONENTS(c) if (v.has_field(c)) {
+    FOR_MAGNETIC_COMPONENTS(c) if (gv.has_field(c)) {
       if (data.ninvmu == 3) abort("more than 3 field components??");
       data.invmu_cs[data.ninvmu] = c;
       data.invmu_ds[data.ninvmu] = component_direction(c);
       ++data.ninvmu;
     }
   
-  data.offsets = new int[2 * num_fields];
-  for (int i = 0; i < 2 * num_fields; ++i)
+  data.offsets = new int[2 * num_fvals];
+  for (int i = 0; i < 2 * num_fvals; ++i)
     data.offsets[i] = 0;
 
   loop_in_chunks(integrate_chunkloop, (void *) &data, where, cgrid);
 
   delete[] data.offsets;
-  delete[] data.fields;
+  delete[] data.fvals;
   delete[] data.ph;
   delete[] data.cS;
 
@@ -203,45 +204,45 @@ complex<double> fields::integrate(int num_fields, const component *components,
 typedef struct { 
   field_rfunction integrand; void *integrand_data; 
 } rfun_wrap_data;
-static complex<double> rfun_wrap(const complex<double> *fields,
+static complex<double> rfun_wrap(const complex<double> *fvals,
 				 const vec &loc, void *data_) {
   rfun_wrap_data *data = (rfun_wrap_data *) data_;
-  return data->integrand(fields, loc, data->integrand_data);
+  return data->integrand(fvals, loc, data->integrand_data);
 }
 
-double fields::integrate(int num_fields, const component *components,
+double fields::integrate(int num_fvals, const component *components,
 			 field_rfunction integrand,
 			 void *integrand_data_,
-			 const geometric_volume &where,
+			 const volume &where,
 			 double *maxabs)
 {
   rfun_wrap_data data;
   data.integrand = integrand;
   data.integrand_data = integrand_data_;
-  return real(integrate(num_fields, components, rfun_wrap,
+  return real(integrate(num_fvals, components, rfun_wrap,
 			&data, where, maxabs));
 }
 
-double fields::max_abs(int num_fields, const component *components,
+double fields::max_abs(int num_fvals, const component *components,
 		       field_function integrand,
 		       void *integrand_data_,
-		       const geometric_volume &where)
+		       const volume &where)
 {
   double maxabs;
-  integrate(num_fields, components, integrand, integrand_data_, where,
+  integrate(num_fvals, components, integrand, integrand_data_, where,
 	    &maxabs);
   return maxabs;
 }
 
-double fields::max_abs(int num_fields, const component *components,
+double fields::max_abs(int num_fvals, const component *components,
 		       field_rfunction integrand,
 		       void *integrand_data_,
-		       const geometric_volume &where)
+		       const volume &where)
 {
   rfun_wrap_data data;
   data.integrand = integrand;
   data.integrand_data = integrand_data_;
-  return max_abs(num_fields, components, rfun_wrap, &data, where);
+  return max_abs(num_fvals, components, rfun_wrap, &data, where);
 }
 
 static complex<double> return_the_field(const complex<double> *fields,
@@ -252,7 +253,7 @@ static complex<double> return_the_field(const complex<double> *fields,
   return fields[0];
 }
 
-double fields::max_abs(int c, const geometric_volume &where)
+double fields::max_abs(int c, const volume &where)
 {
   if (is_derived(c))
     return max_abs(derived_component(c), where);
@@ -260,20 +261,20 @@ double fields::max_abs(int c, const geometric_volume &where)
     return max_abs(component(c), where);
 }
 
-double fields::max_abs(component c, const geometric_volume &where)
+double fields::max_abs(component c, const volume &where)
 {
   if (is_derived(int(c)))
     return max_abs(derived_component(c), where);
   return max_abs(1, &c, return_the_field, 0, where);
 }
 
-double fields::max_abs(derived_component c, const geometric_volume &where)
+double fields::max_abs(derived_component c, const volume &where)
 {
   if (!is_derived(int(c)))
     return max_abs(component(c), where);
   int nfields;
   component cs[12];
-  field_rfunction fun = derived_component_func(c, v, nfields, cs);
+  field_rfunction fun = derived_component_func(c, gv, nfields, cs);
   return max_abs(nfields, cs, fun, &nfields, where);
 }
 
