@@ -1,5 +1,5 @@
 // -*- C++ -*-
-/* Copyright (C) 2005-2008 Massachusetts Institute of Technology
+/* Copyright (C) 2005-2014 Massachusetts Institute of Technology
 %
 %  This program is free software; you can redistribute it and/or modify
 %  it under the terms of the GNU General Public License as published by
@@ -26,14 +26,15 @@ using namespace std;
 namespace meep {
 
 const int NUM_FIELD_COMPONENTS = 20;
-const int NUM_FIELD_TYPES = 5;
+const int NUM_FIELD_TYPES = 8;
 
 enum component { Ex=0, Ey, Er, Ep, Ez, Hx, Hy, Hr, Hp, Hz,
                  Dx, Dy, Dr, Dp, Dz, Bx, By, Br, Bp, Bz, Dielectric, Permeability };
+#define Centered Dielectric // better name for centered "dielectric" grid
 enum derived_component { Sx=100, Sy, Sr, Sp, Sz, EnergyDensity,
 			 D_EnergyDensity, H_EnergyDensity };
 enum ndim { D1=0, D2, D3, Dcyl };
-enum field_type { E_stuff=0, H_stuff=1, D_stuff=2, B_stuff=3, P_stuff=4 };
+enum field_type { E_stuff=0, H_stuff=1, D_stuff=2, B_stuff=3, PE_stuff=4, PH_stuff=5, WE_stuff=6, WH_stuff=7 };
 enum boundary_side { High=0, Low };
 enum direction { X=0,Y,Z,R,P, NO_DIRECTION };
 struct signed_direction {
@@ -67,101 +68,168 @@ inline direction stop_at_direction(ndim dim) {
 
 component first_field_component(field_type ft);
 
-#define FOR_FIELD_TYPES(ft) for (field_type ft = E_stuff; \
-                                 ft <= P_stuff; ft = (field_type) (ft+1))
-#define FOR_ELECTRIC_COMPONENTS(c) for (component c = Ex; \
-                                        c < Hx; c = (component) (c+1))
-#define FOR_MAGNETIC_COMPONENTS(c) for (component c = Hz; \
-                                        c > Ez; c = (component) (c-1))
-#define FOR_B_COMPONENTS(c) for (component c = Bz; \
-				 c > Dz; c = (component) (c-1))
-#define FOR_H_AND_B(h,b) for (component h=Hx, b=Bx; \
-			      h <= Hz; h = (component) (h+1), b = (component) (b+1))
-#define FOR_D_COMPONENTS(c) for (component c = Dz; \
-                                 c > Hz; c = (component) (c-1))
-#define FOR_E_AND_D(e,d) for (component e = Ex, d = Dx; \
-                              e <= Ez; e = (component) (e+1), d = (component) (d+1))
-#define FOR_FT_COMPONENTS(ft,c) for (component c = first_field_component(ft), loop_cstop = component(first_field_component(ft) + NUM_FIELD_TYPES); c < loop_cstop; c = component(c+1))
-#define FOR_COMPONENTS(c) for (component c = Ex,loop_stop_co=Ey; \
-                               c != loop_stop_co; \
-                               c = (component)((c+1)%NUM_FIELD_COMPONENTS), \
-                               loop_stop_co = Ex)
-#define FOR_DIRECTIONS(d) for (direction d = X,loop_stop_di=Y; \
-                               d != loop_stop_di; \
-                               d = (direction)((d+1)%5), \
-                               loop_stop_di = X)
-#define FOR_SIDES(s) for (boundary_side s = High, loop_stop_bi=Low; \
-			  s != loop_stop_bi; \
-			  s = (boundary_side) ((s+1) % 2), \
-			  loop_stop_bi = High)
+#define FOR_FIELD_TYPES(ft) for (meep::field_type ft = meep::E_stuff;	\
+				 ft <= meep::WH_stuff;			\
+                                 ft = (meep::field_type) (ft+1))
+#define FOR_ELECTRIC_COMPONENTS(c) for (meep::component c = meep::Ex;	\
+                                        c < meep::Hx;			\
+                                        c = (meep::component) (c+1))
+#define FOR_MAGNETIC_COMPONENTS(c) for (meep::component c = meep::Hz;	\
+                                        c > meep::Ez;			\
+                                        c = (meep::component) (c-1))
+#define FOR_B_COMPONENTS(c) for (meep::component c = meep::Bz;	\
+				 c > meep::Dz; c = (meep::component) (c-1))
+#define FOR_H_AND_B(h,b) for (meep::component h=meep::Hx, b=meep::Bx;	\
+			      h <= meep::Hz;				\
+                              h = (meep::component) (h+1),		\
+			      b = (meep::component) (b+1))
+#define FOR_D_COMPONENTS(c) for (meep::component c = meep::Dz;	\
+                                 c > meep::Hz; c = (meep::component) (c-1))
+#define FOR_E_AND_D(e,d) for (meep::component e = meep::Ex, d = meep::Dx; \
+                              e <= meep::Ez; e = (meep::component) (e+1), \
+                              d = (component) (d+1))
+#define FOR_E_AND_H(c) for (meep::component c = meep::Ex; c < meep::Dx; \
+                            c = (meep::component) (c+1))
+#define FOR_D_AND_B(c) for (meep::component c = meep::Dx;		\
+                            c < meep::Dielectric; c = (meep::component) (c+1))
+#define FOR_FT_COMPONENTS(ft,c) for (meep::component c = meep::first_field_component(ft), loop_cstop = meep::component(meep::first_field_component(ft) + 5); c < loop_cstop; c = meep::component(c+1))
+#define FOR_COMPONENTS(c) for (meep::component c = meep::Ex,		\
+                                               loop_stop_co = meep::Ey; \
+                               c != loop_stop_co;			\
+                               c = (meep::component)((c+1) %		\
+					       meep::NUM_FIELD_COMPONENTS), \
+			       loop_stop_co = meep::Ex)
+#define FOR_DIRECTIONS(d) for (meep::direction d = meep::X,		\
+                                               loop_stop_di = meep::Y;	\
+                               d != loop_stop_di;			\
+                               d = (meep::direction)((d+1)%5),		\
+                               loop_stop_di = meep::X)
+#define FOR_SIDES(s) for (meep::boundary_side s = meep::High,		\
+                                             loop_stop_bi = meep::Low;	\
+			  s != loop_stop_bi;				\
+                          s = (meep::boundary_side) ((s+1) % 2),	\
+                          loop_stop_bi = meep::High)
 
 // only loop over directions where we have coordinates
-#define LOOP_OVER_DIRECTIONS(dim, d) for (direction d = start_at_direction(dim), \
-                                     loop_stop_directi = stop_at_direction(dim); \
-                                     d < loop_stop_directi; d = (direction) (d+1))
+#define LOOP_OVER_DIRECTIONS(dim, d)					\
+  for (meep::direction d = meep::start_at_direction(dim),		\
+                       loop_stop_directi = meep::stop_at_direction(dim); \
+       d < loop_stop_directi; d = (meep::direction) (d+1))
 
 // loop over all directions in which we might have fields
-#define LOOP_OVER_FIELD_DIRECTIONS(dim, d) for (direction d = dim == Dcyl ? Z : X; d < (dim == Dcyl ? NO_DIRECTION : R); d = direction(d+1))
+#define LOOP_OVER_FIELD_DIRECTIONS(dim, d) for (meep::direction d = dim == meep::Dcyl ? meep::Z : meep::X; d < (dim == meep::Dcyl ? meep::NO_DIRECTION : meep::R); d = meep::direction(d+1))
 
-// loop over indices idx from is to ie (inclusive) in v
-#define LOOP_OVER_IVECS(v, is, ie, idx) \
+// loop over indices idx from is to ie (inclusive) in gv
+#define LOOP_OVER_IVECS(gv, is, ie, idx) \
   for (int loop_is1 = (is).yucky_val(0), \
            loop_is2 = (is).yucky_val(1), \
            loop_is3 = (is).yucky_val(2), \
            loop_n1 = ((ie).yucky_val(0) - loop_is1) / 2 + 1, \
            loop_n2 = ((ie).yucky_val(1) - loop_is2) / 2 + 1, \
            loop_n3 = ((ie).yucky_val(2) - loop_is3) / 2 + 1, \
-           loop_d1 = (v).yucky_direction(0), \
-           loop_d2 = (v).yucky_direction(1), \
-           loop_d3 = (v).yucky_direction(2), \
-           loop_s1 = (v).stride0((direction) loop_d1), \
-           loop_s2 = (v).stride0((direction) loop_d2), \
-           loop_s3 = (v).stride0((direction) loop_d3), \
-           idx0 = (is - (v).little_corner()).yucky_val(0) / 2 * loop_s1 \
-                + (is - (v).little_corner()).yucky_val(1) / 2 * loop_s2 \
-                + (is - (v).little_corner()).yucky_val(2) / 2 * loop_s3,\
+           loop_d1 = (gv).yucky_direction(0), \
+           loop_d2 = (gv).yucky_direction(1), \
+           loop_d3 = (gv).yucky_direction(2), \
+	   loop_s1 = (gv).stride((meep::direction) loop_d1),		\
+	   loop_s2 = (gv).stride((meep::direction) loop_d2),		\
+	   loop_s3 = (gv).stride((meep::direction) loop_d3),		\
+           idx0 = (is - (gv).little_corner()).yucky_val(0) / 2 * loop_s1 \
+                + (is - (gv).little_corner()).yucky_val(1) / 2 * loop_s2 \
+                + (is - (gv).little_corner()).yucky_val(2) / 2 * loop_s3,\
            loop_i1 = 0; loop_i1 < loop_n1; loop_i1++) \
     for (int loop_i2 = 0; loop_i2 < loop_n2; loop_i2++) \
       for (int idx = idx0 + loop_i1*loop_s1 + loop_i2*loop_s2, \
            loop_i3 = 0; loop_i3 < loop_n3; loop_i3++, idx+=loop_s3)
 
-#define LOOP_OVER_VOL(v, c, idx) \
-  LOOP_OVER_IVECS(v, (v).little_corner() + (v).iyee_shift(c), (v).big_corner() + (v).iyee_shift(c), idx)
+#define LOOP_OVER_VOL(gv, c, idx) \
+  LOOP_OVER_IVECS(gv, (gv).little_corner() + (gv).iyee_shift(c), (gv).big_corner() + (gv).iyee_shift(c), idx)
 
-#define LOOP_OVER_VOL_OWNED(v, c, idx) \
-  LOOP_OVER_IVECS(v, (v).little_owned_corner(c), (v).big_corner(), idx)
+#define LOOP_OVER_VOL_OWNED(gv, c, idx) \
+  LOOP_OVER_IVECS(gv, (gv).little_owned_corner(c), (gv).big_corner(), idx)
 
-#define LOOP_OVER_VOL_OWNED0(v, c, idx) \
-  LOOP_OVER_IVECS(v, (v).little_owned_corner0(c), (v).big_corner(), idx)
+#define LOOP_OVER_VOL_OWNED0(gv, c, idx) \
+  LOOP_OVER_IVECS(gv, (gv).little_owned_corner0(c), (gv).big_corner(), idx)
 
-#define LOOP_OVER_VOL_NOTOWNED(v, c, idx) \
- for (ivec loop_notowned_is((v).dim,0), loop_notowned_ie((v).dim,0); \
-      loop_notowned_is == zero_ivec((v).dim);) \
-   for (int loop_ibound = 0; (v).get_boundary_icorners(c, loop_ibound,     \
+#define LOOP_OVER_VOL_NOTOWNED(gv, c, idx) \
+ for (ivec loop_notowned_is((gv).dim,0), loop_notowned_ie((gv).dim,0); \
+      loop_notowned_is == zero_ivec((gv).dim);) \
+   for (int loop_ibound = 0; (gv).get_boundary_icorners(c, loop_ibound,     \
 		  				       &loop_notowned_is,  \
 						       &loop_notowned_ie); \
 	loop_ibound++) \
-     LOOP_OVER_IVECS(v, loop_notowned_is, loop_notowned_ie, idx)
+     LOOP_OVER_IVECS(gv, loop_notowned_is, loop_notowned_ie, idx)
+
+#define LOOPS_ARE_STRIDE1(gv) ((gv).stride((gv).yucky_direction(2)) == 1)
+
+// The following work identically to the LOOP_* macros above,
+// but assume that the inner loop is stride-1: LOOPS_ARE_STRIDE1(gv) *must*
+// be true.  These are useful in allowing gcc to auto-vectorize the inner
+// loop, since gcc's vectorizer requires the array stride to be known at
+// compile time.  Note that stride-1 loops are the most common case in Meep.
+// Note that we also specify _Pragma("ivdep"), which is a hint to
+// compilers like icc (and hopefully gcc at some point) that the loop
+// iterations don't have data dependencies.  This means that you
+// should only use these macros where that is true!  (Basically,
+// all of this is here to support performance hacks of step_generic.)
+
+// loop over indices idx from is to ie (inclusive) in gv
+#define S1LOOP_OVER_IVECS(gv, is, ie, idx) \
+  for (int loop_is1 = (is).yucky_val(0), \
+           loop_is2 = (is).yucky_val(1), \
+           loop_is3 = (is).yucky_val(2), \
+           loop_n1 = ((ie).yucky_val(0) - loop_is1) / 2 + 1, \
+           loop_n2 = ((ie).yucky_val(1) - loop_is2) / 2 + 1, \
+           loop_n3 = ((ie).yucky_val(2) - loop_is3) / 2 + 1, \
+           loop_d1 = (gv).yucky_direction(0), \
+           loop_d2 = (gv).yucky_direction(1), \
+	   loop_s1 = (gv).stride((meep::direction) loop_d1),	\
+	   loop_s2 = (gv).stride((meep::direction) loop_d2),	\
+           loop_s3 = 1, \
+           idx0 = (is - (gv).little_corner()).yucky_val(0) / 2 * loop_s1 \
+                + (is - (gv).little_corner()).yucky_val(1) / 2 * loop_s2 \
+                + (is - (gv).little_corner()).yucky_val(2) / 2 * loop_s3,\
+           loop_i1 = 0; loop_i1 < loop_n1; loop_i1++) \
+    for (int loop_i2 = 0; loop_i2 < loop_n2; loop_i2++) _Pragma("ivdep") \
+      for (int idx = idx0 + loop_i1*loop_s1 + loop_i2*loop_s2, \
+           loop_i3 = 0; loop_i3 < loop_n3; loop_i3++, idx++)
+
+#define S1LOOP_OVER_VOL(gv, c, idx) \
+  S1LOOP_OVER_IVECS(gv, (gv).little_corner() + (gv).iyee_shift(c), (gv).big_corner() + (gv).iyee_shift(c), idx)
+
+#define S1LOOP_OVER_VOL_OWNED(gv, c, idx) \
+  S1LOOP_OVER_IVECS(gv, (gv).little_owned_corner(c), (gv).big_corner(), idx)
+
+#define S1LOOP_OVER_VOL_OWNED0(gv, c, idx) \
+  S1LOOP_OVER_IVECS(gv, (gv).little_owned_corner0(c), (gv).big_corner(), idx)
+
+#define S1LOOP_OVER_VOL_NOTOWNED(gv, c, idx) \
+ for (ivec loop_notowned_is((gv).dim,0), loop_notowned_ie((gv).dim,0); \
+      loop_notowned_is == meep::zero_ivec((gv).dim);)			\
+   for (int loop_ibound = 0; (gv).get_boundary_icorners(c, loop_ibound,     \
+		  				       &loop_notowned_is,  \
+						       &loop_notowned_ie); \
+	loop_ibound++) \
+     S1LOOP_OVER_IVECS(gv, loop_notowned_is, loop_notowned_ie, idx)
 
 #define IVEC_LOOP_AT_BOUNDARY 					\
  ((loop_s1 != 0 && (loop_i1 == 0 || loop_i1 == loop_n1-1)) ||	\
   (loop_s2 != 0 && (loop_i2 == 0 || loop_i2 == loop_n2-1)) ||	\
   (loop_s3 != 0 && (loop_i3 == 0 || loop_i3 == loop_n3-1)))
 
-#define IVEC_LOOP_ILOC(v, iloc) \
-  ivec iloc((v).dim); \
+#define IVEC_LOOP_ILOC(gv, iloc) \
+  ivec iloc((gv).dim); \
   iloc.set_direction(direction(loop_d1), loop_is1 + 2*loop_i1); \
   iloc.set_direction(direction(loop_d2), loop_is2 + 2*loop_i2); \
   iloc.set_direction(direction(loop_d3), loop_is3 + 2*loop_i3)
 
-#define IVEC_LOOP_LOC(v, loc) \
-  vec loc((v).dim); \
-  loc.set_direction(direction(loop_d1), (0.5*loop_is1 + loop_i1) * (v).inva); \
-  loc.set_direction(direction(loop_d2), (0.5*loop_is2 + loop_i2) * (v).inva); \
-  loc.set_direction(direction(loop_d3), (0.5*loop_is3 + loop_i3) * (v).inva)
+#define IVEC_LOOP_LOC(gv, loc) \
+  vec loc((gv).dim); \
+  loc.set_direction(direction(loop_d1), (0.5*loop_is1 + loop_i1) * (gv).inva); \
+  loc.set_direction(direction(loop_d2), (0.5*loop_is2 + loop_i2) * (gv).inva); \
+  loc.set_direction(direction(loop_d3), (0.5*loop_is3 + loop_i3) * (gv).inva)
 
 // integration weight for using LOOP_OVER_IVECS with field::integrate
-#define IVEC_LOOP_WEIGHT1x(s0, s1, e0, e1, i, n, dir) ((i > 1 && i < n - 2) ? 1.0 : (i == 0 ? (s0).in_direction(direction(dir)) : (i == 1 ? (s1).in_direction(direction(dir)) : i == n - 1 ? (e0).in_direction(direction(dir)) : (i == n - 2 ? (e1).in_direction(direction(dir)) : 1.0))))
+#define IVEC_LOOP_WEIGHT1x(s0, s1, e0, e1, i, n, dir) ((i > 1 && i < n - 2) ? 1.0 : (i == 0 ? (s0).in_direction(meep::direction(dir)) : (i == 1 ? (s1).in_direction(meep::direction(dir)) : i == n - 1 ? (e0).in_direction(meep::direction(dir)) : (i == n - 2 ? (e1).in_direction(meep::direction(dir)) : 1.0))))
 #define IVEC_LOOP_WEIGHT1(s0, s1, e0, e1, k) IVEC_LOOP_WEIGHT1x(s0, s1, e0, e1, loop_i##k,loop_n##k,loop_d##k)
 #define IVEC_LOOP_WEIGHT(s0, s1, e0, e1, dV) (IVEC_LOOP_WEIGHT1(s0, s1, e0, e1, 3) * (IVEC_LOOP_WEIGHT1(s0, s1, e0, e1, 2) * ((dV) * IVEC_LOOP_WEIGHT1(s0, s1, e0, e1, 1))))
 
@@ -188,6 +256,8 @@ inline bool coordinate_mismatch(ndim dim, direction d) {
 	   (dim == Dcyl && d != R && d != P && d != Z)));
 }
 
+bool is_tm(component c);
+
 extern void abort(const char *, ...); // mympi.cpp
 
 inline bool is_electric(component c) { return c < Hx; }
@@ -211,6 +281,19 @@ const char *component_name(int c);
 const char *direction_name(direction);
 const char *dimension_name(ndim);
 
+inline int component_index(component c) {
+  switch (c) {    
+  case Ex: case Hx: case Dx: case Bx: return 0;
+  case Ey: case Hy: case Dy: case By: return 1;
+  case Ez: case Hz: case Dz: case Bz: return 2;
+  case Er: case Hr: case Dr: case Br: return 0;
+  case Ep: case Hp: case Dp: case Bp: return 1;
+  case Dielectric: return -1;
+  case Permeability: return -1;
+  }
+  return -2; // This code is never reached...
+}
+
 direction component_direction(int c);
 int direction_component(int c, direction d);
 inline direction component_direction(component c) {
@@ -284,6 +367,11 @@ inline int direction_component(int c, direction d) {
     return int(direction_component(component(c), d));
 }
 
+inline component field_type_component(field_type ft, component c) {
+  return direction_component(first_field_component(ft),
+			     component_direction(c));
+}
+
 inline bool coordinate_mismatch(ndim dim, component c) {
   return coordinate_mismatch(dim, component_direction(c));
 }
@@ -391,31 +479,37 @@ class vec {
 
   double project_to_boundary(direction, double boundary_loc);
   friend vec zero_vec(ndim);
+  friend vec one_vec(ndim);
  private:
   double t[5];
 };
 
-inline double abs(const vec &v) { return sqrt(v & v); }
+inline double abs(const vec &pt) { return sqrt(pt & pt); }
 
 inline vec zero_vec(ndim di) {
-  vec v(di); LOOP_OVER_DIRECTIONS(di, d) v.set_direction(d, 0.0);
-  return v;
+  vec pt(di); LOOP_OVER_DIRECTIONS(di, d) pt.set_direction(d, 0.0);
+  return pt;
+}
+
+inline vec one_vec(ndim di) {
+  vec pt(di); LOOP_OVER_DIRECTIONS(di, d) pt.set_direction(d, 1.0);
+  return pt;
 }
 
 inline vec unit_vec(ndim di, direction d) {
-  vec v(zero_vec(di));
-  v.set_direction(d, 1.0);
-  return v;
+  vec pt(zero_vec(di));
+  pt.set_direction(d, 1.0);
+  return pt;
 }
 
-inline vec clean_vec(const vec &v, double val_unused = 0.0) {
-  vec vc(v.dim, val_unused);
-  LOOP_OVER_DIRECTIONS(v.dim, d) vc.set_direction(d, v.in_direction(d));
-  return vc;
+inline vec clean_vec(const vec &pt, double val_unused = 0.0) {
+  vec ptc(pt.dim, val_unused);
+  LOOP_OVER_DIRECTIONS(pt.dim, d) ptc.set_direction(d, pt.in_direction(d));
+  return ptc;
 }
 
 inline vec veccyl(double rr, double zz) {
-  vec v(Dcyl); v.t[R] = rr; v.t[Z] = zz; return v;
+  vec pt(Dcyl); pt.t[R] = rr; pt.t[Z] = zz; return pt;
 }
 
 class ivec;
@@ -531,37 +625,37 @@ class ivec {
 };
 
 inline ivec zero_ivec(ndim di) {
-  ivec v; v.dim = di; LOOP_OVER_DIRECTIONS(di, d) v.set_direction(d, 0);
-  return v;
+  ivec pt; pt.dim = di; LOOP_OVER_DIRECTIONS(di, d) pt.set_direction(d, 0);
+  return pt;
 }
 
 inline ivec one_ivec(ndim di) {
-  ivec v; v.dim = di; LOOP_OVER_DIRECTIONS(di, d) v.set_direction(d, 1);
-  return v;
+  ivec pt; pt.dim = di; LOOP_OVER_DIRECTIONS(di, d) pt.set_direction(d, 1);
+  return pt;
 }
 
 inline ivec unit_ivec(ndim di, direction d) {
-  ivec v(zero_ivec(di));
-  v.set_direction(d, 1);
-  return v;
+  ivec pt(zero_ivec(di));
+  pt.set_direction(d, 1);
+  return pt;
 }
 
 inline ivec iveccyl(int rr, int zz) {
-  ivec v(Dcyl); v.t[R] = rr; v.t[Z] = zz; return v;
+  ivec pt(Dcyl); pt.t[R] = rr; pt.t[Z] = zz; return pt;
 }
 
 vec max(const vec &vec1, const vec &vec2);
 vec min(const vec &vec1, const vec &vec2);
 ivec max(const ivec &ivec1, const ivec &ivec2);
 ivec min(const ivec &ivec1, const ivec &ivec2);
-ivec max_to_all(const ivec &v); // in mympi.cpp
+ivec max_to_all(const ivec &); // in mympi.cpp
 
-class geometric_volume {
+class volume {
  public:
   ndim dim;
-  geometric_volume(ndim di) { dim = di; min_corner.dim = di; max_corner.dim = di; };
-  geometric_volume(const vec &vec1, const vec &vec2);
-  geometric_volume(const vec &pt);
+  volume(ndim di) { dim = di; min_corner.dim = di; max_corner.dim = di; };
+  volume(const vec &vec1, const vec &vec2);
+  volume(const vec &pt);
   void set_direction_min(direction d, double val) { min_corner.set_direction(d, val); };
   void set_direction_max(direction d, double val) { max_corner.set_direction(d, val); };
   double in_direction_min(direction d) const { return min_corner.in_direction(d); };
@@ -573,37 +667,38 @@ class geometric_volume {
   vec center() const { return (min_corner + max_corner) * 0.5; }
   double diameter() const;
   bool contains(const vec &h) const;
-  geometric_volume intersect_with(const geometric_volume &a) const;
-  geometric_volume operator&(const geometric_volume &a) const {
+  bool contains(const volume &a) const;
+  volume intersect_with(const volume &a) const;
+  volume operator&(const volume &a) const {
     return intersect_with(a);
   };
-  geometric_volume operator|(const geometric_volume &a) const {
-    return geometric_volume(min(min_corner, a.min_corner),
+  volume operator|(const volume &a) const {
+    return volume(min(min_corner, a.min_corner),
 			    max(max_corner, a.max_corner));
   };
-  geometric_volume operator+(const vec &a) const {
-    return geometric_volume(min_corner + a, max_corner + a);
+  volume operator+(const vec &a) const {
+    return volume(min_corner + a, max_corner + a);
   }
-  geometric_volume operator+=(const vec &a) {
+  volume operator+=(const vec &a) {
     min_corner += a; max_corner += a;
     return *this;
   }
-  geometric_volume operator-(const vec &a) const {
-    return geometric_volume(min_corner - a, max_corner - a);
+  volume operator-(const vec &a) const {
+    return volume(min_corner - a, max_corner - a);
   }
-  geometric_volume operator-=(const vec &a) {
+  volume operator-=(const vec &a) {
     min_corner -= a; max_corner -= a;
     return *this;
   }
-  bool operator==(const geometric_volume &a) const {
+  bool operator==(const volume &a) const {
     return (min_corner == a.min_corner && max_corner == a.max_corner);
   }
-  bool operator!=(const geometric_volume &a) const { return !(*this == a); };
-  geometric_volume round_float(void) const {
-    return geometric_volume(min_corner.round_float(),max_corner.round_float());
+  bool operator!=(const volume &a) const { return !(*this == a); };
+  volume round_float(void) const {
+    return volume(min_corner.round_float(),max_corner.round_float());
   }
-  bool intersects(const geometric_volume &a) const;
-  bool operator&&(const geometric_volume &a) const {
+  bool intersects(const volume &a) const;
+  bool operator&&(const volume &a) const {
     return intersects(a);
   };
   vec get_min_corner() const { return min_corner; };
@@ -613,24 +708,23 @@ class geometric_volume {
   vec min_corner, max_corner;
 };
 
-class volume;
-volume volcyl(double rsize, double zsize, double a);
-volume volone(double zsize, double a);
-volume vol1d(double zsize, double a);
-volume voltwo(double xsize, double ysize, double a);
-volume vol2d(double xsize, double ysize, double a);
-volume vol3d(double xsize, double ysize, double zsize, double a);
+class grid_volume;
+grid_volume volcyl(double rsize, double zsize, double a);
+grid_volume volone(double zsize, double a);
+grid_volume vol1d(double zsize, double a);
+grid_volume voltwo(double xsize, double ysize, double a);
+grid_volume vol2d(double xsize, double ysize, double a);
+grid_volume vol3d(double xsize, double ysize, double zsize, double a);
 
-class volume {
+class grid_volume {
  public:
-  volume() {};
+  grid_volume() {};
 
   ndim dim;
   double a, inva /* = 1/a */;
 
   void print() const;
   int stride(direction d) const { return the_stride[d]; };
-  int stride0(direction d) const { return the_stride0[d]; };
   int num_direction(direction d) const {
     return num[((int) d) % 3];
   };
@@ -664,9 +758,9 @@ class volume {
   void interpolate(component, const vec &, int indices[8], double weights[8]) const;
   void interpolate(component, const vec &, ivec locs[8], double weights[8]) const;
 
-  geometric_volume dV(component c, int index) const;
-  geometric_volume dV(const ivec &, double diameter = 1.0) const;
-  bool intersect_with(const volume &vol_in, volume *intersection = NULL, volume *others = NULL, int *num_others = NULL) const;
+  volume dV(component c, int index) const;
+  volume dV(const ivec &, double diameter = 1.0) const;
+  bool intersect_with(const grid_volume &vol_in, grid_volume *intersection = NULL, grid_volume *others = NULL, int *num_others = NULL) const;
   double rmin() const;
   double rmax() const;
   double xmin() const;
@@ -690,7 +784,8 @@ class volume {
   }
   vec yee_shift(component) const;
   component eps_component() const;
-  void yee2diel_offsets(component c, int &offset1, int &offset2);
+  void yee2cent_offsets(component c, int &offset1, int &offset2) const;
+  void cent2yee_offsets(component c, int &offset1, int &offset2) const;
 
   double boundary_location(boundary_side, direction) const;
   ivec big_corner() const;
@@ -708,29 +803,29 @@ class volume {
 
   ivec little_owned_corner(component c) const;
   bool owns(const ivec &) const;
-  geometric_volume surroundings() const;
-  geometric_volume interior() const;
+  volume surroundings() const;
+  volume interior() const;
 
   bool get_boundary_icorners(component c, int ib, ivec *cs, ivec *ce) const;
 
-  friend volume volcyl(double rsize, double zsize, double a);
-  friend volume volone(double zsize, double a);
-  friend volume vol1d(double zsize, double a);
-  friend volume voltwo(double xsize, double ysize, double a);
-  friend volume vol2d(double xsize, double ysize, double a);
-  friend volume vol3d(double xsize, double ysize, double zsize, double a);
+  friend grid_volume volcyl(double rsize, double zsize, double a);
+  friend grid_volume volone(double zsize, double a);
+  friend grid_volume vol1d(double zsize, double a);
+  friend grid_volume voltwo(double xsize, double ysize, double a);
+  friend grid_volume vol2d(double xsize, double ysize, double a);
+  friend grid_volume vol3d(double xsize, double ysize, double zsize, double a);
 
-  volume split(int num, int which) const;
-  volume split_by_effort(int num, int which, int Ngv = 0, const volume *gv = NULL, double *effort = NULL) const;
-  volume split_at_fraction(bool want_high, int numer) const;
-  volume halve(direction d) const;
+  grid_volume split(int num, int which) const;
+  grid_volume split_by_effort(int num, int which, int Ngv = 0, const grid_volume *v = NULL, double *effort = NULL) const;
+  grid_volume split_at_fraction(bool want_high, int numer) const;
+  grid_volume halve(direction d) const;
   void pad_self(direction d);
-  volume pad(direction d) const;
-  volume pad() const {
-       volume v(*this);
+  grid_volume pad(direction d) const;
+  grid_volume pad() const {
+       grid_volume gv(*this);
        LOOP_OVER_DIRECTIONS(dim,d)
-	    v.pad_self(d);
-       return v;
+	    gv.pad_self(d);
+       return gv;
   }
   ivec iyee_shift(component c) const {
     ivec out = zero_ivec(dim);
@@ -758,24 +853,24 @@ class volume {
   double origin_z() const { return origin.z(); }
 
  private:
-  volume(ndim d, double ta, int na, int nb, int nc);
+  grid_volume(ndim d, double ta, int na, int nb, int nc);
   ivec io; // integer origin ... always change via set_origin etc.!
   vec origin; // cache of operator[](io), for performance
   void update_ntot();
   void set_strides();
   void num_changed() { update_ntot(); set_strides(); }
   int num[3];
-  int the_stride0[5], the_stride[5];
+  int the_stride[5];
   int the_ntot;
 };
 
-class geometric_volume_list;
+class volume_list;
 
 class symmetry;
 symmetry identity();
-symmetry rotate4(direction,const volume &);
-symmetry rotate2(direction,const volume &);
-symmetry mirror(direction,const volume &);
+symmetry rotate4(direction,const grid_volume &);
+symmetry rotate2(direction,const grid_volume &);
+symmetry mirror(direction,const grid_volume &);
 symmetry r_to_minus_r_symmetry(double m);
 
 class symmetry {
@@ -784,15 +879,15 @@ class symmetry {
   symmetry(const symmetry &);
   ~symmetry();
   friend symmetry identity();
-  friend symmetry rotate4(direction,const volume &);
-  friend symmetry rotate2(direction,const volume &);
-  friend symmetry mirror(direction,const volume &);
+  friend symmetry rotate4(direction,const grid_volume &);
+  friend symmetry rotate2(direction,const grid_volume &);
+  friend symmetry mirror(direction,const grid_volume &);
 
   signed_direction transform(direction d, int n) const;
   ivec transform(const ivec &, int n) const;
   vec transform(const vec &, int n) const;
   ivec transform_unshifted(const ivec &, int n) const;
-  geometric_volume transform(const geometric_volume &, int n) const;
+  volume transform(const volume &, int n) const;
   component transform(component, int n) const;
   complex<double> phase_shift(component, int n) const;
   derived_component transform(derived_component, int n) const;
@@ -802,7 +897,7 @@ class symmetry {
   int multiplicity() const;
   bool is_primitive(const ivec &) const;
 
-  geometric_volume_list *reduce(const geometric_volume_list *gl) const;
+  volume_list *reduce(const volume_list *gl) const;
 
   symmetry operator+(const symmetry &) const;
   symmetry operator*(complex<double>) const;
@@ -821,15 +916,15 @@ class symmetry {
   friend symmetry r_to_minus_r_symmetry(double m);
 };
 
-class geometric_volume_list {
+class volume_list {
 public:
-  geometric_volume_list(const geometric_volume &gv, int c, complex<double> weight = 1.0, geometric_volume_list *next = 0) : gv(gv), c(c), weight(weight), next(next) {}
-  ~geometric_volume_list() { delete next; }
+  volume_list(const volume &v, int c, complex<double> weight = 1.0, volume_list *next = 0) : v(v), c(c), weight(weight), next(next) {}
+  ~volume_list() { delete next; }
   
-  geometric_volume gv;
-  int c; // component or derived component associated with gv (e.g. for flux)
+  volume v;
+  int c; // component or derived component associated with v (e.g. for flux)
   complex<double> weight;
-  geometric_volume_list *next;
+  volume_list *next;
 };
 
 } /* namespace meep */
