@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2008 Massachusetts Institute of Technology
+/* Copyright (C) 2005-2014 Massachusetts Institute of Technology
 %
 %  This program is free software; you can redistribute it and/or modify
 %  it under the terms of the GNU General Public License as published by
@@ -44,44 +44,6 @@ inline int rmin_bulk(int m) {
   return r;
 }
 
-class polarizability {
- public:
-  volume v;
-  polarizability(const structure_chunk *, material_function &sig,
-                 double om, double ga, double sigscale,
-                 double energy_saturation = 0.0, bool mine = true);
-  polarizability(const polarizability *);
-  ~polarizability();
-  double gamma, omeganot, *sigma, *s[NUM_FIELD_COMPONENTS];
-  double energy_saturation, saturated_sigma;
-  bool is_mine() { return is_it_mine; };
-  bool is_it_mine;
-  polarizability *next;
-
-  polarizability_identifier get_identifier() const;
-};
-
-class polarization {
- public:
-  polarization(const polarizability *the_pb, int is_real, bool store_enrgy=0);
-  ~polarization();
-  double saturation_factor;
-  double *(P[NUM_FIELD_COMPONENTS][2]), *(energy[NUM_FIELD_COMPONENTS]),
-    *(s[NUM_FIELD_COMPONENTS]);
-  int is_real;
-  bool store_energy;
-  const polarizability *pb;
-  polarization *next;
-
-  complex<double> analytic_epsilon(double freq, const vec &) const;
-  double local_energy(const ivec &);
-  // for total energy, use fields::thermo_energy_in_box
-  static polarization *set_up_polarizations(const structure_chunk *s, int is_real, bool store_enrgy = 0);
-  void use_real_fields();
-  void zero_fields();
-  void initialize_energy(double energy(const vec &));
-};
-
 class src_vol {
  public:
   src_vol(component cc, src_time *st, int n, int *ind, complex<double> *amps);
@@ -140,25 +102,74 @@ symmetry r_to_minus_r_symmetry(int m);
 
 // functions in step_generic.cpp:
 
-void step_curl(double *f, component c, const double *g1, const double *g2,
+void step_curl(realnum *f, component c, const realnum *g1, const realnum *g2,
 	       int s1, int s2, // strides for g1/g2 shift
-	       const volume &v, double dtdx,
-	       direction dsig, const double *sig, const double *siginv,
-	       double dt, const double *cnd, const double *cndinv);
+	       const grid_volume &gv, double dtdx,
+	       direction dsig, const double *sig, const double *kap, const double *siginv,
+	       realnum *fu, direction dsigu, const double *sigu, const double *kapu, const double *siginvu,
+	       double dt, const realnum *cnd, const realnum *cndinv,
+	       realnum *fcnd);
 
-void step_update_EDHB(double *f, component fc, const volume &v,
-		      const double *g, const double *g1, const double *g2,
-		      const double *gb, const double *g1b, const double *g2b,
-		      const double *u, const double *u1, const double *u2,
+void step_update_EDHB(realnum *f, component fc, const grid_volume &gv,
+		      const realnum *g, const realnum *g1, const realnum *g2,
+		      const realnum *u, const realnum *u1, const realnum *u2,
 		      int s, int s1, int s2,
-		      const double *chi2, const double *chi3,
-		      direction dsig,const double *sig,const double *siginv,
-		      direction dsigg, const double *sigg,
-		      direction dsig1, const double *sig1,
-		      direction dsig1inv, const double *sig1inv,
-		      direction dsig2, const double *sig2,
-		      direction dsig2inv, const double *sig2inv,
-		      int sigsize_dsig,int sigsize_dsigg,int sigsize_dsig1);
+		      const realnum *chi2, const realnum *chi3,
+		      realnum *fw, direction dsigw, const double *sigw, const double *kapw);
 
+void step_beta(realnum *f, component c, const realnum *g,
+	       const grid_volume &gv, double betadt,
+	       direction dsig, const double *siginv,
+	       realnum *fu, direction dsigu, const double *siginvu,
+	       const realnum *cndinv, realnum *fcnd);
+
+// functions in step_generic_stride1.cpp, generated from step_generic.cpp:
+
+void step_curl_stride1(realnum *f, component c, const realnum *g1, const realnum *g2,
+	       int s1, int s2, // strides for g1/g2 shift
+	       const grid_volume &gv, double dtdx,
+	       direction dsig, const double *sig, const double *kap, const double *siginv,
+	       realnum *fu, direction dsigu, const double *sigu, const double *kapu, const double *siginvu,
+	       double dt, const realnum *cnd, const realnum *cndinv,
+               realnum *fcnd);
+
+void step_update_EDHB_stride1(realnum *f, component fc, const grid_volume &gv,
+		      const realnum *g, const realnum *g1, const realnum *g2,
+		      const realnum *u, const realnum *u1, const realnum *u2,
+		      int s, int s1, int s2,
+		      const realnum *chi2, const realnum *chi3,
+		      realnum *fw, direction dsigw, const double *sigw, const double *kapw);
+
+void step_beta_stride1(realnum *f, component c, const realnum *g,
+		       const grid_volume &gv, double betadt,
+		       direction dsig, const double *siginv,
+		       realnum *fu, direction dsigu, const double *siginvu,
+		       const realnum *cndinv, realnum *fcnd);
+
+/* macro wrappers around time-stepping functions: for performance reasons,
+   if the inner loop is stride-1 then we use the stride-1 versions,
+   which allow gcc (and possibly other compilers) to do additional
+   optimizations, especially loop vectorization */
+
+#define STEP_CURL(f, c, g1, g2, s1, s2, gv, dtdx, dsig, sig, kap, siginv, fu, dsigu, sigu, kapu, siginvu, dt, cnd, cndinv, fcnd) do { \
+  if (LOOPS_ARE_STRIDE1(gv))						\
+    step_curl_stride1(f, c, g1, g2, s1, s2, gv, dtdx, dsig, sig, kap, siginv, fu, dsigu, sigu, kapu, siginvu, dt, cnd, cndinv, fcnd); \
+  else									\
+    step_curl(f, c, g1, g2, s1, s2, gv, dtdx, dsig, sig, kap, siginv, fu, dsigu, sigu, kapu, siginvu, dt, cnd, cndinv, fcnd); \
+} while (0)
+
+#define STEP_UPDATE_EDHB(f, fc, gv, g, g1, g2, u, u1, u2, s, s1, s2, chi2, chi3, fw, dsigw, sigw, kapw) do { \
+  if (LOOPS_ARE_STRIDE1(gv))						\
+    step_update_EDHB_stride1(f, fc, gv, g, g1, g2, u, u1, u2, s, s1, s2, chi2, chi3, fw, dsigw, sigw, kapw); \
+  else									\
+    step_update_EDHB(f, fc, gv, g, g1, g2, u, u1, u2, s, s1, s2, chi2, chi3, fw, dsigw, sigw, kapw); \
+} while (0)
+
+#define STEP_BETA(f, c, g, gv, betadt, dsig, siginv, fu, dsigu, siginvu, cndinv, fcnd) do {	\
+  if (LOOPS_ARE_STRIDE1(gv))						\
+    step_beta_stride1(f, c,g, gv, betadt, dsig, siginv, fu, dsigu, siginvu, cndinv, fcnd);	\
+  else									\
+    step_beta(f, c,g, gv, betadt, dsig, siginv, fu, dsigu, siginvu, cndinv, fcnd);		\
+} while (0)
 
 } // namespace meep
