@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2008 Massachusetts Institute of Technology
+/* Copyright (C) 2005-2014 Massachusetts Institute of Technology
 %
 %  This program is free software; you can redistribute it and/or modify
 %  it under the terms of the GNU General Public License as published by
@@ -23,7 +23,13 @@
 #include "config.h"
 
 #ifdef HAVE_MPI
-#include <mpi.h>
+#  ifdef NEED_UNDEF_SEEK_FOR_MPI
+// undef'ing SEEK_* is needed for MPICH, possibly other MPI versions
+#    undef SEEK_SET
+#    undef SEEK_END
+#    undef SEEK_CUR
+#  endif
+#  include <mpi.h>
 #endif
 
 #ifdef IGNORE_SIGFPE
@@ -59,8 +65,14 @@ extern "C" int feenableexcept (int EXCEPTS);
 
 #define UNUSED(x) (void) x // silence compiler warnings
 
+#define MPI_REALNUM (sizeof(realnum) == sizeof(double) ? MPI_DOUBLE:MPI_FLOAT)
+
 namespace meep {
 
+#ifdef HAVE_MPI
+  static MPI_Comm mycomm = MPI_COMM_WORLD;
+#endif
+
 bool quiet = false; // defined in meep.h
 
 initialize::initialize(int &argc, char** &argv) {
@@ -86,6 +98,7 @@ initialize::initialize(int &argc, char** &argv) {
 initialize::~initialize() {
   if (!quiet) master_printf("\nElapsed run time = %g s\n", elapsed_time());
 #ifdef HAVE_MPI
+  end_divide_parallel();
   MPI_Finalize();
 #endif
 }
@@ -120,9 +133,9 @@ void send(int from, int to, double *data, int size) {
   if (from == to) return;
   if (size == 0) return;
   const int me = my_rank();
-  if (from == me) MPI_Send(data, size, MPI_DOUBLE, to, 1, MPI_COMM_WORLD);
+  if (from == me) MPI_Send(data, size, MPI_DOUBLE, to, 1, mycomm);
   MPI_Status stat;
-  if (to == me) MPI_Recv(data, size, MPI_DOUBLE, from, 1, MPI_COMM_WORLD, &stat);
+  if (to == me) MPI_Recv(data, size, MPI_DOUBLE, from, 1, mycomm, &stat);
 #else
   UNUSED(from);
   UNUSED(to);
@@ -131,10 +144,23 @@ void send(int from, int to, double *data, int size) {
 #endif
 }
 
+#if MEEP_SINGLE
+void broadcast(int from, realnum *data, int size) {
+#ifdef HAVE_MPI
+  if (size == 0) return;
+  MPI_Bcast(data, size, MPI_FLOAT, from, mycomm);
+#else
+  UNUSED(from);
+  UNUSED(data);
+  UNUSED(size);
+#endif
+}
+#endif
+
 void broadcast(int from, double *data, int size) {
 #ifdef HAVE_MPI
   if (size == 0) return;
-  MPI_Bcast(data, size, MPI_DOUBLE, from, MPI_COMM_WORLD);
+  MPI_Bcast(data, size, MPI_DOUBLE, from, mycomm);
 #else
   UNUSED(from);
   UNUSED(data);
@@ -145,7 +171,7 @@ void broadcast(int from, double *data, int size) {
 void broadcast(int from, char *data, int size) {
 #ifdef HAVE_MPI
   if (size == 0) return;
-  MPI_Bcast(data, size, MPI_CHAR, from, MPI_COMM_WORLD);
+  MPI_Bcast(data, size, MPI_CHAR, from, mycomm);
 #else
   UNUSED(from);
   UNUSED(data);
@@ -156,7 +182,7 @@ void broadcast(int from, char *data, int size) {
 void broadcast(int from, complex<double> *data, int size) {
 #ifdef HAVE_MPI
   if (size == 0) return;
-  MPI_Bcast(data, 2*size, MPI_DOUBLE, from, MPI_COMM_WORLD);
+  MPI_Bcast(data, 2*size, MPI_DOUBLE, from, mycomm);
 #else
   UNUSED(from);
   UNUSED(data);
@@ -167,7 +193,7 @@ void broadcast(int from, complex<double> *data, int size) {
 void broadcast(int from, int *data, int size) {
 #ifdef HAVE_MPI
   if (size == 0) return;
-  MPI_Bcast(data, size, MPI_INT, from, MPI_COMM_WORLD);
+  MPI_Bcast(data, size, MPI_INT, from, mycomm);
 #else
   UNUSED(from);
   UNUSED(data);
@@ -177,7 +203,7 @@ void broadcast(int from, int *data, int size) {
 
 complex<double> broadcast(int from, complex<double> data) {
 #ifdef HAVE_MPI
-  MPI_Bcast(&data, 2, MPI_DOUBLE, from, MPI_COMM_WORLD);
+  MPI_Bcast(&data, 2, MPI_DOUBLE, from, mycomm);
 #else
   UNUSED(from);
 #endif
@@ -186,7 +212,7 @@ complex<double> broadcast(int from, complex<double> data) {
 
 double broadcast(int from, double data) {
 #ifdef HAVE_MPI
-  MPI_Bcast(&data, 1, MPI_DOUBLE, from, MPI_COMM_WORLD);
+  MPI_Bcast(&data, 1, MPI_DOUBLE, from, mycomm);
 #else
   UNUSED(from);
 #endif
@@ -195,7 +221,7 @@ double broadcast(int from, double data) {
 
 int broadcast(int from, int data) {
 #ifdef HAVE_MPI
-  MPI_Bcast(&data, 1, MPI_INT, from, MPI_COMM_WORLD);
+  MPI_Bcast(&data, 1, MPI_INT, from, mycomm);
 #else
   UNUSED(from);
 #endif
@@ -209,7 +235,7 @@ bool broadcast(int from, bool b) {
 double max_to_master(double in) {
   double out = in;
 #ifdef HAVE_MPI
-  MPI_Reduce(&in,&out,1,MPI_DOUBLE,MPI_MAX,0,MPI_COMM_WORLD);
+  MPI_Reduce(&in,&out,1,MPI_DOUBLE,MPI_MAX,0,mycomm);
 #endif
   return out;
 }
@@ -217,7 +243,7 @@ double max_to_master(double in) {
 double max_to_all(double in) {
   double out = in;
 #ifdef HAVE_MPI
-  MPI_Allreduce(&in,&out,1,MPI_DOUBLE,MPI_MAX,MPI_COMM_WORLD);
+  MPI_Allreduce(&in,&out,1,MPI_DOUBLE,MPI_MAX,mycomm);
 #endif
   return out;
 }
@@ -225,26 +251,26 @@ double max_to_all(double in) {
 int max_to_all(int in) {
   int out = in;
 #ifdef HAVE_MPI
-  MPI_Allreduce(&in,&out,1,MPI_INT,MPI_MAX,MPI_COMM_WORLD);
+  MPI_Allreduce(&in,&out,1,MPI_INT,MPI_MAX,mycomm);
 #endif
   return out;
 }
 
-ivec max_to_all(const ivec &v) {
+ivec max_to_all(const ivec &pt) {
   int in[5], out[5];
-  for (int i=0; i<5; ++i) in[i] = out[i] = v.in_direction(direction(i));
+  for (int i=0; i<5; ++i) in[i] = out[i] = pt.in_direction(direction(i));
 #ifdef HAVE_MPI
-  MPI_Allreduce(&in,&out,5,MPI_INT,MPI_MAX,MPI_COMM_WORLD);
+  MPI_Allreduce(&in,&out,5,MPI_INT,MPI_MAX,mycomm);
 #endif
-  ivec vout(v.dim);
-  for (int i=0; i<5; ++i) vout.set_direction(direction(i), out[i]);
-  return vout;
+  ivec ptout(pt.dim);
+  for (int i=0; i<5; ++i) ptout.set_direction(direction(i), out[i]);
+  return ptout;
 }
 
 double sum_to_master(double in) {
   double out = in;
 #ifdef HAVE_MPI
-  MPI_Reduce(&in,&out,1,MPI_DOUBLE,MPI_SUM,0,MPI_COMM_WORLD);
+  MPI_Reduce(&in,&out,1,MPI_DOUBLE,MPI_SUM,0,mycomm);
 #endif
   return out;
 }
@@ -252,26 +278,41 @@ double sum_to_master(double in) {
 double sum_to_all(double in) {
   double out = in;
 #ifdef HAVE_MPI
-  MPI_Allreduce(&in,&out,1,MPI_DOUBLE,MPI_SUM,MPI_COMM_WORLD);
+  MPI_Allreduce(&in,&out,1,MPI_DOUBLE,MPI_SUM,mycomm);
 #endif
   return out;
 }
 
 void sum_to_all(const double *in, double *out, int size) {
 #ifdef HAVE_MPI
-  MPI_Allreduce((void*) in, out, size, MPI_DOUBLE,MPI_SUM,MPI_COMM_WORLD);
+  MPI_Allreduce((void*) in, out, size, MPI_DOUBLE,MPI_SUM,mycomm);
 #else
   memcpy(out, in, sizeof(double) * size);
 #endif
 }
 
+void sum_to_all(const float *in, double *out, int size) {
+  double *in2 = new double[size];
+  for (int i = 0; i < size; ++i) in2[i] = in[i];
+  sum_to_all(in2, out, size);
+  delete[] in2;
+}
+
+void sum_to_all(const complex<double> *in, complex<double> *out, int size) {
+  sum_to_all((const double*) in, (double*) out, 2*size);
+}
+
+void sum_to_all(const complex<float> *in, complex<double> *out, int size) {
+  sum_to_all((const float*) in, (double*) out, 2*size);
+}
+
 long double sum_to_all(long double in) {
   long double out = in;
 #ifdef HAVE_MPI
   if (MPI_LONG_DOUBLE == MPI_DATATYPE_NULL)
     out = sum_to_all(double(in));
   else
-    MPI_Allreduce(&in,&out,1,MPI_LONG_DOUBLE,MPI_SUM,MPI_COMM_WORLD);
+    MPI_Allreduce(&in,&out,1,MPI_LONG_DOUBLE,MPI_SUM,mycomm);
 #endif
   return out;
 }
@@ -279,7 +320,7 @@ long double sum_to_all(long double in) {
 int sum_to_all(int in) {
   int out = in;
 #ifdef HAVE_MPI
-  MPI_Allreduce(&in,&out,1,MPI_INT,MPI_SUM,MPI_COMM_WORLD);
+  MPI_Allreduce(&in,&out,1,MPI_INT,MPI_SUM,mycomm);
 #endif
   return out;
 }
@@ -287,7 +328,7 @@ int sum_to_all(int in) {
 int partial_sum_to_all(int in) {
   int out = in;
 #ifdef HAVE_MPI
-  MPI_Scan(&in,&out,1,MPI_INT,MPI_SUM,MPI_COMM_WORLD);
+  MPI_Scan(&in,&out,1,MPI_INT,MPI_SUM,mycomm);
 #endif
   return out;
 }
@@ -295,7 +336,7 @@ int partial_sum_to_all(int in) {
 complex<double> sum_to_all(complex<double> in) {
   complex<double> out = in;
 #ifdef HAVE_MPI
-  MPI_Allreduce(&in,&out,2,MPI_DOUBLE,MPI_SUM,MPI_COMM_WORLD);
+  MPI_Allreduce(&in,&out,2,MPI_DOUBLE,MPI_SUM,mycomm);
 #endif
   return out;
 }
@@ -309,7 +350,7 @@ complex<long double> sum_to_all(complex<long double> in) {
     out = complex<long double>(dout.real(), dout.imag());
   }
   else
-    MPI_Allreduce(&in,&out,2,MPI_LONG_DOUBLE,MPI_SUM,MPI_COMM_WORLD);
+    MPI_Allreduce(&in,&out,2,MPI_LONG_DOUBLE,MPI_SUM,mycomm);
 #endif
   return out;
 }
@@ -317,7 +358,7 @@ complex<long double> sum_to_all(complex<long double> in) {
 bool or_to_all(bool in) {
   int in2 = in, out;
 #ifdef HAVE_MPI
-  MPI_Allreduce(&in2,&out,1,MPI_INT,MPI_LOR,MPI_COMM_WORLD);
+  MPI_Allreduce(&in2,&out,1,MPI_INT,MPI_LOR,mycomm);
 #else
   out = in2;
 #endif
@@ -326,7 +367,7 @@ bool or_to_all(bool in) {
 
 void or_to_all(const int *in, int *out, int size) {
 #ifdef HAVE_MPI
-  MPI_Allreduce((void*) in, out, size, MPI_INT,MPI_LOR,MPI_COMM_WORLD);
+  MPI_Allreduce((void*) in, out, size, MPI_INT,MPI_LOR,mycomm);
 #else
   memcpy(out, in, sizeof(int) * size);
 #endif
@@ -335,7 +376,7 @@ void or_to_all(const int *in, int *out, int size) {
 bool and_to_all(bool in) {
   int in2 = in, out;
 #ifdef HAVE_MPI
-  MPI_Allreduce(&in2,&out,1,MPI_INT,MPI_LAND,MPI_COMM_WORLD);
+  MPI_Allreduce(&in2,&out,1,MPI_INT,MPI_LAND,mycomm);
 #else
   out = in2;
 #endif
@@ -344,7 +385,7 @@ bool and_to_all(bool in) {
 
 void and_to_all(const int *in, int *out, int size) {
 #ifdef HAVE_MPI
-  MPI_Allreduce((void*) in, out, size, MPI_INT,MPI_LAND,MPI_COMM_WORLD);
+  MPI_Allreduce((void*) in, out, size, MPI_INT,MPI_LAND,mycomm);
 #else
   memcpy(out, in, sizeof(int) * size);
 #endif
@@ -352,14 +393,14 @@ void and_to_all(const int *in, int *out, int size) {
 
 void all_wait() {
 #ifdef HAVE_MPI
-  MPI_Barrier(MPI_COMM_WORLD);
+  MPI_Barrier(mycomm);
 #endif
 }
 
 int my_rank() {
 #ifdef HAVE_MPI
   int rank;
-  MPI_Comm_rank(MPI_COMM_WORLD, &rank);
+  MPI_Comm_rank(mycomm, &rank);
   return rank;
 #else
   return 0;
@@ -369,19 +410,71 @@ int my_rank() {
 int count_processors() {
 #ifdef HAVE_MPI
   int n;
-  MPI_Comm_size(MPI_COMM_WORLD, &n);
+  MPI_Comm_size(mycomm, &n);
   return n;
 #else
   return 1;
 #endif
 }
 
+void fields::boundary_communications(field_type ft) {
+  // Communicate the data around!
+#if 0 // This is the blocking version, which should always be safe!
+  for (int noti=0;noti<num_chunks;noti++)
+    for (int j=0;j<num_chunks;j++) {
+      const int i = (noti+j)%num_chunks;
+      const int pair = j+i*num_chunks;
+      DOCMP {
+        send(chunks[j]->n_proc(), chunks[i]->n_proc(),
+             comm_blocks[ft][pair], comm_size_tot(ft,pair));
+      }
+    }
+#endif
+#ifdef HAVE_MPI
+  const int maxreq = num_chunks*num_chunks;
+  MPI_Request *reqs = new MPI_Request[maxreq];
+  MPI_Status *stats = new MPI_Status[maxreq];
+  int reqnum = 0;
+  int *tagto = new int[count_processors()];
+  for (int i=0;i<count_processors();i++) tagto[i] = 0;
+  for (int noti=0;noti<num_chunks;noti++)
+    for (int j=0;j<num_chunks;j++) {
+      const int i = (noti+j)%num_chunks;
+      const int pair = j+i*num_chunks;
+      const int comm_size = comm_size_tot(ft,pair);
+      if (comm_size > 0) {
+	if (chunks[j]->is_mine() && !chunks[i]->is_mine())
+	  MPI_Isend(comm_blocks[ft][pair], comm_size,
+		    MPI_REALNUM, chunks[i]->n_proc(),
+		    tagto[chunks[i]->n_proc()]++,
+		    mycomm, &reqs[reqnum++]);
+	if (chunks[i]->is_mine() && !chunks[j]->is_mine())
+	  MPI_Irecv(comm_blocks[ft][pair], comm_size,
+		    MPI_REALNUM, chunks[j]->n_proc(),
+		    tagto[chunks[j]->n_proc()]++,
+		    mycomm, &reqs[reqnum++]);
+      }
+    }
+  delete[] tagto;
+  if (reqnum > maxreq) abort("Too many requests!!!\n");
+  if (reqnum > 0) MPI_Waitall(reqnum, reqs, stats);
+  delete[] reqs;
+  delete[] stats;
+#else
+  (void) ft; // unused
+#endif
+}
+
 // IO Routines...
 
+bool am_really_master() {
+  return (my_global_rank() == 0);
+}
+
 void master_printf(const char *fmt, ...) {
   va_list ap;
   va_start(ap, fmt);
-  if (am_master()) { vprintf(fmt, ap); fflush(stdout); }
+  if (am_really_master()) { vprintf(fmt, ap); fflush(stdout); }
   va_end(ap);
 }
 
@@ -437,12 +530,12 @@ void begin_critical_section(int tag)
 {
 #ifdef HAVE_MPI
      int process_rank;
-     MPI_Comm_rank(MPI_COMM_WORLD, &process_rank);
+     MPI_Comm_rank(mycomm, &process_rank);
      if (process_rank > 0) { /* wait for a message before continuing */
 	  MPI_Status status;
 	  int recv_tag = tag - 1; /* initialize to wrong value */
 	  MPI_Recv(&recv_tag, 1, MPI_INT, process_rank - 1, tag, 
-		   MPI_COMM_WORLD, &status);
+		   mycomm, &status);
 	  if (recv_tag != tag) abort("invalid tag received in begin_critical_section");
      }
 #else
@@ -454,15 +547,91 @@ void end_critical_section(int tag)
 {
 #ifdef HAVE_MPI
      int process_rank, num_procs;
-     MPI_Comm_rank(MPI_COMM_WORLD, &process_rank);
-     MPI_Comm_size(MPI_COMM_WORLD, &num_procs);
+     MPI_Comm_rank(mycomm, &process_rank);
+     MPI_Comm_size(mycomm, &num_procs);
      if (process_rank != num_procs - 1) { /* send a message to next process */
 	  MPI_Send(&tag, 1, MPI_INT, process_rank + 1, tag, 
-		   MPI_COMM_WORLD);
+		   mycomm);
      }
 #else
      UNUSED(tag);
 #endif
 }
 
+
+/* Simple, somewhat hackish API to allow user to run multiple simulations
+   in parallel in the same MPI job.  The user calls
+
+   mygroup = divide_parallel_processes(numgroups);
+
+   to divide all of the MPI processes into numgroups equal groups,
+   and to return the index (from 0 to numgroups-1) of the current group.
+   From this point on, all fields etc. that you create and all
+   calls from mympi.cpp will only communicate within your group of
+   processes.
+
+   However, there are two calls that you can use to switch back to
+   globally communication among all processes:
+
+   begin_global_communications();
+   ....do stuff....
+   end_global_communications();
+
+   It is important not to mix the two types; e.g. you cannot timestep
+   a field created in the local group in global mode, or vice versa.
+*/
+
+int divide_parallel_processes(int numgroups)
+{
+#ifdef HAVE_MPI
+  end_divide_parallel();
+  if (numgroups > count_processors()) abort("numgroups > count_processors");
+  int mygroup = (my_rank() * numgroups) / count_processors();
+  MPI_Comm_split(MPI_COMM_WORLD, mygroup, my_rank(), &mycomm);
+  return mygroup;
+#else
+  if (numgroups != 1) abort("cannot divide processes in non-MPI mode");
+  return 0;
+#endif
+}
+
+#ifdef HAVE_MPI
+  static MPI_Comm mycomm_save = MPI_COMM_WORLD;
+#endif
+
+void begin_global_communications(void)
+{
+#ifdef HAVE_MPI
+  mycomm_save = mycomm;
+  mycomm = MPI_COMM_WORLD;
+#endif
+}
+
+void end_global_communications(void)
+{
+#ifdef HAVE_MPI
+  mycomm = mycomm_save;
+  mycomm_save = MPI_COMM_WORLD;
+#endif
+}
+
+void end_divide_parallel(void)
+{
+#ifdef HAVE_MPI
+  if (mycomm != MPI_COMM_WORLD) MPI_Comm_free(&mycomm);
+  if (mycomm_save != MPI_COMM_WORLD) MPI_Comm_free(&mycomm_save);
+  mycomm = mycomm_save = MPI_COMM_WORLD;
+#endif
+}
+
+int my_global_rank() {
+#ifdef HAVE_MPI
+  int rank;
+  MPI_Comm_rank(MPI_COMM_WORLD, &rank);
+  return rank;
+#else
+  return 0;
+#endif
+}
+
 } // namespace meep
