@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2008 Massachusetts Institute of Technology
+/* Copyright (C) 2005-2014 Massachusetts Institute of Technology
 %
 %  This program is free software; you can redistribute it and/or modify
 %  it under the terms of the GNU General Public License as published by
@@ -23,9 +23,6 @@
 #include "meep_internals.hpp"
 
 #include "config.h"
-#ifdef HAVE_MPI
-#include <mpi.h>
-#endif
 
 #define RESTRICT
 
@@ -61,34 +58,29 @@ void fields::step() {
   for (int i=0;i<num_chunks;i++) chunks[i]->s->update_condinv();
 
   calc_sources(time()); // for B sources
-
   step_db(B_stuff);
-  if (!disable_sources) step_b_source();
+  step_source(B_stuff);
   step_boundaries(B_stuff);
-  update_h_from_b();
+  calc_sources(time() + 0.5*dt); // for integrated H sources
+  update_eh(H_stuff);
+  step_boundaries(WH_stuff);
+  update_pols(H_stuff);
+  step_boundaries(PH_stuff);
   step_boundaries(H_stuff);
-  // because step_boundaries overruns the timing stack...
-  am_now_working_on(Stepping);
 
   if (fluxes) fluxes->update_half();
 
   calc_sources(time() + 0.5*dt); // for D sources
-
   step_db(D_stuff);
-  if (!disable_sources) step_d_source();
+  step_source(D_stuff);
   step_boundaries(D_stuff);
-
   calc_sources(time() + dt); // for integrated E sources
-
-  update_e_from_d();
+  update_eh(E_stuff);
+  step_boundaries(WE_stuff);
+  update_pols(E_stuff);
+  step_boundaries(PE_stuff);
   step_boundaries(E_stuff);
 
-  // because step_boundaries overruns the timing stack...
-  am_now_working_on(Stepping);
-
-  update_from_e();
-  step_boundaries(P_stuff);
-
   if (fluxes) fluxes->update();
   t += 1;
   update_dfts();
@@ -104,38 +96,48 @@ void fields::step() {
 double fields_chunk::peek_field(component c, const vec &where) {
   double w[8];
   ivec ilocs[8];
-  v.interpolate(c,where, ilocs, w);
-  if (v.contains(ilocs[0]) && f[c][0]) {
+  gv.interpolate(c,where, ilocs, w);
+  if (gv.contains(ilocs[0]) && f[c][0]) {
     double hello = 0.0;
-    if (is_mine()) hello = f[c][0][v.index(c,ilocs[0])];
+    if (is_mine()) hello = f[c][0][gv.index(c,ilocs[0])];
     broadcast(n_proc(), &hello, 1);
     return hello;
   }
   //abort("Got no such %s field at %g %g!\n",
-  //      component_name(c), v[ilocs[0]].x(), v[ilocs[0]].y());
+  //      component_name(c), gv[ilocs[0]].x(), gv[ilocs[0]].y());
   return 0.0;
 }
 
 void fields::phase_material() {
-  if (phasein_time > 0) {
+  bool changed = false;
+  if (is_phasing()) {
     for (int i=0;i<num_chunks;i++)
-      if (chunks[i]->is_mine())
+      if (chunks[i]->is_mine()) {
 	chunks[i]->phase_material(phasein_time);
-    step_boundaries(E_stuff);
+	changed = changed || chunks[i]->new_s;
+      }
     phasein_time--;
   }
+  if (or_to_all(changed)) {
+    calc_sources(time() + 0.5*dt); // for integrated H sources
+    update_eh(H_stuff); // ensure H = 1/mu * B
+    step_boundaries(H_stuff);
+    calc_sources(time() + dt); // for integrated E sources
+    update_eh(E_stuff); // ensure E = 1/eps * D
+    step_boundaries(E_stuff);
+  }
 }
 
 void fields_chunk::phase_material(int phasein_time) {
   if (new_s && phasein_time > 0) {
     changing_structure();
     s->mix_with(new_s, 1.0/phasein_time);
-    update_e_from_d(); // ensure E = 1/eps * D
   }
 }
 
 void fields::step_boundaries(field_type ft) {
   connect_chunks(); // re-connect if !chunk_connections_valid
+
   am_now_working_on(MpiTime);
 
   // Do the metals first!
@@ -164,49 +166,7 @@ void fields::step_boundaries(field_type ft) {
       }
     }
 
-  // Communicate the data around!
-#if 0 // This is the blocking version, which should always be safe!
-  for (int noti=0;noti<num_chunks;noti++)
-    for (int j=0;j<num_chunks;j++) {
-      const int i = (noti+j)%num_chunks;
-      const int pair = j+i*num_chunks;
-      DOCMP {
-        send(chunks[j]->n_proc(), chunks[i]->n_proc(),
-             comm_blocks[ft][pair], comm_size_tot(ft,pair));
-      }
-    }
-#endif
-#ifdef HAVE_MPI
-  const int maxreq = num_chunks*num_chunks;
-  MPI_Request *reqs = new MPI_Request[maxreq];
-  MPI_Status *stats = new MPI_Status[maxreq];
-  int reqnum = 0;
-  int *tagto = new int[count_processors()];
-  for (int i=0;i<count_processors();i++) tagto[i] = 0;
-  for (int noti=0;noti<num_chunks;noti++)
-    for (int j=0;j<num_chunks;j++) {
-      const int i = (noti+j)%num_chunks;
-      const int pair = j+i*num_chunks;
-      const int comm_size = comm_size_tot(ft,pair);
-      if (comm_size > 0) {
-	if (chunks[j]->is_mine() && !chunks[i]->is_mine())
-	  MPI_Isend(comm_blocks[ft][pair], comm_size,
-		    MPI_DOUBLE, chunks[i]->n_proc(),
-		    tagto[chunks[i]->n_proc()]++,
-		    MPI_COMM_WORLD, &reqs[reqnum++]);
-	if (chunks[i]->is_mine() && !chunks[j]->is_mine())
-	  MPI_Irecv(comm_blocks[ft][pair], comm_size,
-		    MPI_DOUBLE, chunks[j]->n_proc(),
-		    tagto[chunks[j]->n_proc()]++,
-		    MPI_COMM_WORLD, &reqs[reqnum++]);
-      }
-    }
-  delete[] tagto;
-  if (reqnum > maxreq) abort("Too many requests!!!\n");
-  if (reqnum > 0) MPI_Waitall(reqnum, reqs, stats);
-  delete[] reqs;
-  delete[] stats;
-#endif
+  boundary_communications(ft);
   
   // Finally, copy incoming data to the fields themselves, multiplying phases:
   for (int i=0;i<num_chunks;i++)
@@ -239,42 +199,37 @@ void fields::step_boundaries(field_type ft) {
   finished_working();
 }
 
-void fields::step_b_source(int including_integrated) {
+void fields::step_source(field_type ft, bool including_integrated) {
+  if (ft != D_stuff && ft != B_stuff) abort("only step_source(D/B) is okay");
   for (int i=0;i<num_chunks;i++)
     if (chunks[i]->is_mine())
-      chunks[i]->step_b_source(chunks[i]->b_sources, including_integrated);
+      chunks[i]->step_source(ft, including_integrated);
 }
-void fields_chunk::step_b_source(src_vol *sv, int including_integrated) {
-  if (sv == NULL) return;
-  component c = direction_component(Bx, component_direction(sv->c));
-  if (f[c][0] && is_magnetic(sv->c))
-    for (int j=0; j<sv->npts; j++) {
-      const complex<double> A = sv->current(j) * dt;
-      const int i = sv->index[j];
-      f[c][0][i] -= real(A);
-      if (!is_real) f[c][1][i] -= imag(A);
-    }
-  step_b_source(sv->next, including_integrated);
-}
-
-void fields::step_d_source(int including_integrated) {
-  for (int i=0;i<num_chunks;i++)
-    if (chunks[i]->is_mine())
-      chunks[i]->step_d_source(chunks[i]->d_sources, including_integrated);
-}
-void fields_chunk::step_d_source(src_vol *sv, int including_integrated) {
-  if (sv == NULL) return;
-  component c = direction_component(Dx, component_direction(sv->c));
-  if ((including_integrated || !sv->t->is_integrated)
-      && f[c][0] && is_electric(sv->c)) {
-    for (int j=0; j<sv->npts; j++) {
-      const complex<double> A = sv->current(j) * dt;
-      const int i = sv->index[j];
-      f[c][0][i] -= real(A);
-      if (!is_real) f[c][1][i] -= imag(A);
+void fields_chunk::step_source(field_type ft, bool including_integrated) {
+  if (doing_solve_cw && !including_integrated) return;
+  for (src_vol *sv = sources[ft]; sv; sv = sv->next) {
+    component c = direction_component(first_field_component(ft), 
+				      component_direction(sv->c));
+    const realnum *cndinv = s->condinv[c][component_direction(sv->c)];
+    if ((including_integrated || !sv->t->is_integrated)	&& f[c][0]
+	&& ((ft == D_stuff && is_electric(sv->c))
+	    || (ft == B_stuff && is_magnetic(sv->c)))) {
+      if (cndinv)
+	for (int j=0; j<sv->npts; j++) {
+	  const int i = sv->index[j];
+	  const complex<double> A = sv->current(j) * dt * double(cndinv[i]);
+	  f[c][0][i] -= real(A);
+	  if (!is_real) f[c][1][i] -= imag(A);
+	}
+      else
+	for (int j=0; j<sv->npts; j++) {
+	  const complex<double> A = sv->current(j) * dt;
+	  const int i = sv->index[j];
+	  f[c][0][i] -= real(A);
+	  if (!is_real) f[c][1][i] -= imag(A);
+	}
     }
   }
-  step_d_source(sv->next, including_integrated);
 }
 
 void fields::calc_sources(double tim) {
