@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2008 Massachusetts Institute of Technology  
+/* Copyright (C) 2005-2014 Massachusetts Institute of Technology  
 %
 %  This program is free software; you can redistribute it and/or modify
 %  it under the terms of the GNU General Public License as published by
@@ -36,7 +36,8 @@ double rods(const vec &r) {
 }
 
 void compare(double b, double a, const char *n) {
-  if (fabs(a-b) > fabs(b)*1e-5 || b != b) {
+  double thresh = sizeof(realnum) == sizeof(float) ? 1e-4 : 1e-5;
+  if (fabs(a-b) > fabs(b)*thresh || b != b) {
     abort("Failed %s (%g instead of %g, relerr %0.2g)\n", n,
 	  a, b, fabs(a-b)/fabs(b));
   } else {
@@ -46,112 +47,100 @@ void compare(double b, double a, const char *n) {
 
 static double dpml = 1.0;
 
-double using_pml_ez(const volume &v, double eps(const vec &)) {
+double using_pml_ez(const grid_volume &gv, double eps(const vec &)) {
   const double ttot = 30.0;
-  structure s(v, eps, pml(dpml));
+  structure s(gv, eps, pml(dpml));
   fields f(&s);
-  f.add_point_source(Ez, 0.2, 3.0, 0.0, 2.0, v.center(),
+  f.add_point_source(Ez, 0.2, 3.0, 0.0, 2.0, gv.center(),
 		     complex<double>(0,-2*pi*0.2));
-  while (f.time() < ttot) f.step();
+  while (f.round_time() < ttot) f.step();
   monitor_point p;
-  f.get_point(&p, v.center());
+  f.get_point(&p, gv.center());
   return real(p.get_component(Ez));
 }
 
-double x_periodic_y_pml(const volume &v, double eps(const vec &)) {
+double x_periodic_y_pml(const grid_volume &gv, double eps(const vec &)) {
   const double ttot = 30.0;
-  structure s(v, eps, pml(dpml, Y));
+  structure s(gv, eps, pml(dpml, Y));
   fields f(&s);
-  f.add_point_source(Ez, 0.2, 3.0, 0.0, 2.0, v.center(),
+  f.add_point_source(Ez, 0.2, 3.0, 0.0, 2.0, gv.center(),
 		     complex<double>(0,-2*pi*0.2));
   f.use_bloch(X, 0.1);
-  while (f.time() < ttot) f.step();
+  while (f.round_time() < ttot) f.step();
   monitor_point p;
-  f.get_point(&p, v.center());
+  f.get_point(&p, gv.center());
   return real(p.get_component(Ez));
 }
 
-double x_periodic(const volume &v, double eps(const vec &)) {
+double x_periodic(const grid_volume &gv, double eps(const vec &)) {
   const double ttot = 30.0;
-  structure s(v, eps);
+  structure s(gv, eps);
   fields f(&s);
-  f.add_point_source(Ez, 0.2, 3.0, 0.0, 2.0, v.center(),
+  f.add_point_source(Ez, 0.2, 3.0, 0.0, 2.0, gv.center(),
 		     complex<double>(0,-2*pi*0.2));
   f.use_bloch(X, 0.1);
-  while (f.time() < ttot) f.step();
+  while (f.round_time() < ttot) f.step();
   monitor_point p;
-  f.get_point(&p, v.center());
+  f.get_point(&p, gv.center());
   return real(p.get_component(Ez));
 }
 
-double periodic_ez(const volume &v, double eps(const vec &)) {
+double periodic_ez(const grid_volume &gv, double eps(const vec &)) {
   const double ttot = 30.0;
-  structure s(v, eps);
+  structure s(gv, eps);
   fields f(&s);
-  f.add_point_source(Ez, 0.2, 3.0, 0.0, 2.0, v.center(),
+  f.add_point_source(Ez, 0.2, 3.0, 0.0, 2.0, gv.center(),
 		     complex<double>(0,-2*pi*0.2));
   vec k;
-  switch (v.dim) {
+  switch (gv.dim) {
   case D1: k = vec(0.3); break;
   case D2: k = vec(0.3,0.4); break;
   case D3: k = vec(0.3,0.5,0.8); break;
   case Dcyl: k = veccyl(0.3,0.2); break;
   }
   f.use_bloch(k);
-  while (f.time() < ttot) f.step();
+  while (f.round_time() < ttot) f.step();
   monitor_point p;
-  f.get_point(&p, v.center());
+  f.get_point(&p, gv.center());
   return real(p.get_component(Ez));
 }
 
-double metallic_ez(const volume &v, double eps(const vec &)) {
+double metallic_ez(const grid_volume &gv, double eps(const vec &)) {
   const double ttot = 10.0;
-  structure s(v, eps);
+  structure s(gv, eps);
   fields f(&s);
-  f.add_point_source(Ez, 0.2, 3.0, 0.0, 2.0, v.center(),
+  f.add_point_source(Ez, 0.2, 3.0, 0.0, 2.0, gv.center(),
 		     complex<double>(0,-2*pi*0.2));
-  while (f.time() < ttot) f.step();
+  while (f.round_time() < ttot) f.step();
   monitor_point p;
-  f.get_point(&p, v.center());
+  f.get_point(&p, gv.center());
   return real(p.get_component(Ez));
 }
 
-double polariton_ex(const volume &v, double eps(const vec &)) {
+double sigma(const vec &) { return 7.63; }
+
+double polariton_ex(const grid_volume &gv, double eps(const vec &)) {
   const double ttot = 10.0;
-  structure s(v, eps);
-  s.add_polarizability(one, 0.3, 0.1, 7.63);
+  structure s(gv, eps);
+  s.add_susceptibility(sigma, E_stuff, lorentzian_susceptibility(0.3, 0.1));
   fields f(&s);
-  f.add_point_source(Ex, 0.2, 3.0, 0.0, 2.0, v.center(),
+  f.add_point_source(Ex, 0.2, 3.0, 0.0, 2.0, gv.center(),
 		     complex<double>(0,-2*pi*0.2));
-  while (f.time() < ttot) f.step();
+  while (f.round_time() < ttot) f.step();
   monitor_point p;
-  f.get_point(&p, v.center());
+  f.get_point(&p, gv.center());
   return real(p.get_component(Ex));
 }
 
-double polariton_energy(const volume &v, double eps(const vec &)) {
+double polariton_energy(const grid_volume &gv, double eps(const vec &)) {
   const double ttot = 10.0;
-  structure s(v, eps);
-  s.add_polarizability(one, 0.3, 0.1, 7.63);
-  fields f(&s, 0, 1);
-  f.add_point_source(Ex, 0.2, 3.0, 0.0, 2.0, v.center(),
+  structure s(gv, eps);
+  s.add_susceptibility(sigma, E_stuff, lorentzian_susceptibility(0.3, 0.1));
+  fields f(&s, 0);
+  f.add_point_source(Ex, 0.2, 3.0, 0.0, 2.0, gv.center(),
 		     complex<double>(0,-2*pi*0.2));
-  while (f.time() < ttot) f.step();
-  return f.total_energy();
-}
-
-double saturated_polariton_ex(const volume &v, double eps(const vec &)) {
-  const double ttot = 10.0;
-  structure s(v, eps);
-  polarizability_identifier thep = s.add_polarizability(one, 0.3, 0.1, -0.063, 0.1);
-  fields f(&s);
-  f.use_real_fields();
-  f.add_point_source(Ex, 0.2, 3.0, 0.0, 2.0, v.center(), 
-		     1/sqrt(4*pi) * complex<double>(0,-2*pi*0.2));
-  while (f.time() < ttot) f.step();
-  monitor_point p;
-  f.get_point(&p, v.center());
-  return real(p.get_component(Ex)) * sqrt(4*pi);
+  while (f.round_time() < ttot) f.step();
+  return f.field_energy();
 }
 
 int main(int argc, char **argv) {
@@ -164,16 +153,7 @@ int main(int argc, char **argv) {
 
   compare(-0.0894851, polariton_ex(volone(1.0, a), one),
           "1D polariton");
-#ifdef WITH_SATURABLE_ABSORBERS
-  compare(-0.0384049, saturated_polariton_ex(volone(1.0, a), one),
-          "1D saturated polariton");
-  if (count_processors() <= 2) // FIXME: broken for NCPUS > 2 ???
-    compare(-7.57915, saturated_polariton_ex(vol2d(1.0,1.0, a), one),
-	    "2D saturated polariton");
-  compare(-23.8506, saturated_polariton_ex(vol3d(1.0,1.0,0.5, a), one),
-          "3D saturated polariton");
-#endif
-  compare(0.32617294, polariton_energy(volone(1.0, a), one),
+  compare(0.0863443, polariton_energy(volone(1.0, a), one),
           "1D polariton energy");
   compare(5.20605, metallic_ez(voltwo(1.0, 1.0, a), one),
           "1x1 metallic 2D TM");
@@ -181,7 +161,7 @@ int main(int argc, char **argv) {
           "1x1 PML 2D TM");
   compare(0.110425, x_periodic(voltwo(1.0, 1.0, a), one),
           "1x1 X periodic 2D TM");
-  compare(-4.66027, periodic_ez(voltwo(1.0, 3.0, a), rods),
+  compare(-4.78767, periodic_ez(voltwo(1.0, 3.0, a), rods),
           "1x1 fully periodic 2D TM rods");
   compare(1.12502, periodic_ez(voltwo(1.0, 3.0, a), one),
           "1x1 fully periodic 2D TM");
@@ -193,8 +173,8 @@ int main(int argc, char **argv) {
           "1x1x1 X periodic 3D");
   compare(-101.398, x_periodic_y_pml(vol3d(1.0, 1.0+2*dpml, 1.0, a), one),
           "1x1x1 X periodic Y PML 3D");
-  compare(-97.7989, periodic_ez(vol3d(1.0, 1.0, 1.0, a), rods),
-          "1x1x1 fully periodic 3D rods");
+  compare(-103.844, periodic_ez(vol3d(1.0, 1.0, 1.0, a), rods),
+	  "1x1x1 fully periodic 3D rods");
   compare(-99.1618, periodic_ez(vol3d(1.0, 1.0, 1.0, a), one),
           "1x1x1 fully periodic 3D");
 
